@@ -6,7 +6,8 @@ import sys
 import weakref
 
 from . import version
-from .util import find_library
+from .util import find_library, find_libc
+from .errors import *
 
 XT_INV_PROTO = 0x40  # invert the sense of PROTO
 
@@ -790,24 +791,36 @@ class xtables_target(ct.Union):
                 ("v11", _xtables_target_v11),
                 ("v12", _xtables_target_v12)]
 
-class XTablesError(Exception):
-    """Raised when an xtables call fails for some reason."""
 
-
-_libc, _ = find_library("c")
+_libc = find_libc()
 _optind = ct.c_long.in_dll(_libc, "optind")
 _optarg = ct.c_char_p.in_dll(_libc, "optarg")
 
-_lib_xtables, xtables_version = find_library("xtables")
+xtables_version = os.getenv("PYTHON_IPTABLES_XTABLES_VERSION")
+if xtables_version:
+    _searchlib = "libxtables.so.%s" % (xtables_version,)
+else:
+    _searchlib = "xtables"
+_lib_xtables, xtables_version = find_library(_searchlib)
 _xtables_libdir = os.getenv("XTABLES_LIBDIR")
 if _xtables_libdir is None:
-    import os.path
-    for xtdir in ["/lib/xtables", "/lib64/xtables", "/usr/lib/xtables",
-                  "/usr/lib/iptables", "/usr/lib64/xtables",
-                  "/usr/lib64/iptables", "/usr/local/lib/xtables"]:
-        if os.path.isdir(xtdir):
-            _xtables_libdir = xtdir
-            break
+    import re
+    ldconfig_path_regex = re.compile('^(/.*):$')
+    import subprocess
+    ldconfig = subprocess.Popen(
+        ('/sbin/ldconfig', '-N', '-v'),
+        stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True
+    )
+    ldconfig_out, ldconfig_err = ldconfig.communicate()
+    if ldconfig.returncode != 0:
+        raise XTablesError("ldconfig failed, please set XTABLES_LIBDIR")
+    for ldconfig_out_line in ldconfig_out.splitlines():
+        ldconfig_path_regex_match = ldconfig_path_regex.match(ldconfig_out_line)
+        if ldconfig_path_regex_match is not None:
+            ldconfig_path = os.path.join(ldconfig_path_regex_match.group(1), 'xtables')
+            if os.path.isdir(ldconfig_path):
+                _xtables_libdir = ldconfig_path
+                break
 if _xtables_libdir is None:
     raise XTablesError("can't find directory with extensions; "
                        "please set XTABLES_LIBDIR")
@@ -1007,10 +1020,6 @@ class xtables(object):
         if ext is not None:
             return ext
 
-        xtables._xtables_matches.value = ct.c_void_p(None).value
-        if xtables._xtables_pending_matches:
-            xtables._xtables_pending_matches.value = ct.c_void_p(None).value
-
         match = xtables._xtables_find_match(name, XTF_TRY_LOAD, None)
         if not match:
             self._try_register(name)
@@ -1032,10 +1041,6 @@ class xtables(object):
         if ext is not None:
             return ext
 
-        xtables._xtables_targets.value = ct.c_void_p(None).value
-        if xtables._xtables_pending_targets:
-            xtables._xtables_pending_targets.value = ct.c_void_p(None).value
-
         target = xtables._xtables_find_target(name, XTF_TRY_LOAD)
         if not target:
             self._try_register(name)
